#include "LinuxInput.hpp"

#include "quark/Application.hpp"
#include <GLFW/glfw3.h>

namespace qk {
	Input* Input::instance = new LinuxInput;

	bool LinuxInput::IntIsKeyDown(int keyCode) {
		auto window = std::any_cast<GLFWwindow*>(Application::GetInstance().GetWindow().GetNativeWindow());

		int state = glfwGetKey(window, keyCode);
		return (state == GLFW_PRESS || state == GLFW_REPEAT);
	}

	bool LinuxInput::IntIsMouseDown(int button) {
		auto window = std::any_cast<GLFWwindow*>(Application::GetInstance().GetWindow().GetNativeWindow());

		int state = glfwGetMouseButton(window, button);
		return state == GLFW_PRESS;
	}

	std::pair<float, float> LinuxInput::IntGetMousePosition() {
		auto window = std::any_cast<GLFWwindow*>(Application::GetInstance().GetWindow().GetNativeWindow());
		double x, y;

		glfwGetCursorPos(window, &x, &y);
		return std::make_pair(x, y);
	}
}
