#pragma once

#include <GLFW/glfw3.h>
#include <any>
#include <memory>
#include "quark/Window.hpp"
#include "quark/photon/RenderingContext.hpp"

namespace qk {
	class LinuxWindow :
		public Window
	{
	public:
		LinuxWindow(const WindowConfig& config);
		virtual ~LinuxWindow();

		void OnUpdate() override;

		inline unsigned int GetWidth() const override { return data.width; }
		inline unsigned int GetHeight() const override { return data.height; }

		inline void SetEventCallback(const EventCallbackFn& callback) override { data.eventCallback = callback; }
		constexpr void SetVSync(bool enable) override;
		bool HasVSync() const override;

		virtual std::any GetNativeWindow() const override {
			return window;
		}

	private:
		void Init(const WindowConfig& config);
		void Destroy();

		void InstallGLFWCallbacks();

	private:
		GLFWwindow* window;
		std::unique_ptr<RenderingContext> context;

		struct WindowData {
			std::string title;
			unsigned int width, height;
			bool vsync;

			EventCallbackFn eventCallback;
		} data;
	};
}
