#include "Application.hpp"
#include "quark/Core.hpp"
#include "quark/Input.hpp"
#include "quark/imgui/ImGuiLayer.hpp"
#include <quark.hpp>
#include <spdlog/spdlog.h>

namespace qk {

	Application::Application() {
		QK_CORE_ASSERT(instance == nullptr, "Application already exists");
		instance = this;

		window = Window::Create();
		window->SetEventCallback(BIND_EVENT_CB(Application::OnEvent));

		uiLayer = std::make_unique<ImGuiLayer>();
	}

	void Application::Run() {
		uiLayer->OnAttach();

		while(isRunning) {
			for (Layer* layer : layers) {
				layer->OnUpdate();
			}

			uiLayer->Begin();
			uiLayer->OnImGuiRender();
			for (Layer* layer : layers) {
				layer->OnImGuiRender();
			}
			uiLayer->End();

			window->OnUpdate();
		}

		uiLayer->OnDetach();
	}

	void Application::OnEvent(Event& e) {
		// QK_CORE_TRACE(e);

		EventDispatcher dispatcher(e);
		dispatcher.Dispatch<WindowCloseEvent>(std::bind(&Application::OnWindowClose, this, std::placeholders::_1));

		for (auto it = layers.rbegin(); it != layers.rend(); it++) {
			(*it)->OnEvent(e);
			if (e.isHandled) {
				break;
			}
		}
	}

	void Application::PushLayer(Layer* layer) {
		layers.PushLayer(layer);
		layer->OnAttach();
	}

	void Application::PushOverlay(Layer* layer) {
		layers.PushOverlay(layer);
		layer->OnAttach();
	}

	bool Application::OnWindowClose(WindowCloseEvent& e) {
		isRunning = false;
		return true;
	}
}
