#pragma once

#include "quark/events/Event.hpp"
#include <any>

namespace qk {
	struct WindowConfig {
		std::string title;
		unsigned int width, height;

		WindowConfig(const std::string& title = "quark Engine", unsigned int width = 1280, unsigned int height = 720) :
			title(title), width(width), height(height)
		{ }
	};

	class Window {
	public:
		using EventCallbackFn = std::function<void(Event&)>;

		virtual ~Window() {}

		virtual void OnUpdate() = 0;

		virtual unsigned int GetWidth() const = 0;
		virtual unsigned int GetHeight() const = 0;

		virtual void SetEventCallback(const EventCallbackFn& callback) = 0;
		virtual void SetVSync(bool enabled) = 0;
		virtual bool HasVSync() const = 0;

		virtual std::any GetNativeWindow() const = 0;

		static std::unique_ptr<Window> Create(const WindowConfig& config = WindowConfig());
	};
}
