#pragma once
// IWYU pragma: private, include <quark.hpp>

#include "Event.hpp"

namespace qk {

	class KeyEvent :
		public Event
	{
	public:
		inline int GetKeyCode() const { return keyCode; }

		EVENT_CLASS_CATEGORY(EventCategoryKeyboard | EventCategoryInput)

	protected:
		KeyEvent(int keyCode) :
			keyCode(keyCode)
		{ }

		int keyCode;
	};

	class KeyPressedEvent
		: public KeyEvent
	{
	public:
		KeyPressedEvent(int keyCode, int repeatCount) :
			KeyEvent(keyCode), repeatCount(repeatCount)
		{ }

		virtual operator std::string() const override {
			std::stringstream ss;
			ss << "KeyPressedEvent: " << keyCode << " (x" << repeatCount << ")";
			return ss.str();
		}

		inline int GetRepeatCount() const { return repeatCount; }

		EVENT_CLASS_TYPE(KeyPressed)
	private:
		int repeatCount;
	};

	class KeyTypedEvent
		: public KeyEvent
	{
	public:
		KeyTypedEvent(int keyCode) :
			KeyEvent(keyCode)
		{ }

		virtual operator std::string() const override {
			std::stringstream ss;
			ss << "KeyTypedEvent: " << keyCode;
			return ss.str();
		}

		EVENT_CLASS_TYPE(KeyTyped)
	private:
	};


	class KeyReleasedEvent
		: public KeyEvent
	{
	public:
		KeyReleasedEvent(int keyCode) :
			KeyEvent(keyCode)
		{ }

		virtual operator std::string() const override {
			std::stringstream ss;
			ss << "KeyReleasedEvent: " << keyCode;
			return ss.str();
		}

		EVENT_CLASS_TYPE(KeyReleased)
	};
}
