#pragma once
// IWYU pragma: private, include <quark.hpp>

#include "Event.hpp"

namespace qk {
	class MouseMovedEvent :
		public Event
	{
	public:
		MouseMovedEvent(float x, float y) :
			mouseX(x), mouseY(y)
		{ }

		inline float GetX() const { return mouseX; }
		inline float GetY() const { return mouseY; }

		virtual operator std::string() const override {
			std::stringstream ss;
			ss << "MouseMovedEvent: " << mouseX << ", " << mouseY;
			return ss.str();
		}

		EVENT_CLASS_TYPE(MouseMoved)
		EVENT_CLASS_CATEGORY(EventCategoryMouse | EventCategoryInput)

	private:
		float mouseX, mouseY;
	};

	class MouseScrolledEvent :
		public Event
	{
	public:
		MouseScrolledEvent(float offsetX, float offsetY) :
			offsetX(offsetX), offsetY(offsetY)
		{ }

		inline float GetX() const { return offsetX; }
		inline float GetY() const { return offsetY; }

		virtual operator std::string() const override {
			std::stringstream ss;
			ss << "MouseScrolledEvent: " << offsetX << ", " << offsetY;
			return ss.str();
		}

		EVENT_CLASS_TYPE(MouseScrolled)
		EVENT_CLASS_CATEGORY(EventCategoryMouse | EventCategoryInput)
		
	private:
		float offsetX, offsetY;
	};

	class MouseButtonEvent :
		public Event
	{
	public:
		inline int GetMouseButton() const { return button; }

		EVENT_CLASS_CATEGORY(EventCategoryMouseButton | EventCategoryInput)

	protected:
		MouseButtonEvent(int button) :
			button(button)
		{ }

		int button;
	};

	class MouseButtonPressedEvent :
		public MouseButtonEvent
	{
	public:
		MouseButtonPressedEvent(int button) :
			MouseButtonEvent(button)
		{ }

		virtual operator std::string() const override {
			std::stringstream ss;
			ss << "MouseButtonPressedEvent: " << button;
			return ss.str();
		}

		EVENT_CLASS_TYPE(MouseButtonPressed)
	};

	class MouseButtonReleasedEvent :
		public MouseButtonEvent
	{
	public:
		MouseButtonReleasedEvent(int button) :
			MouseButtonEvent(button)
		{ }

		virtual operator std::string() const override {
			std::stringstream ss;
			ss << "MouseButtonReleasedEvent: " << button;
			return ss.str();
		}

		EVENT_CLASS_TYPE(MouseButtonReleased)
	};
}
